<?php


namespace FiloBlu\Refilo\Remote\Entity\Provider;


use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Framework\App\ResourceConnection;
use FiloBlu\Refilo\Remote\Entity\Category;
use Zend_Db;

class CategoryPermissionsProvider implements EntityProviderInterface
{

    /**
     * @var int
     */
    protected $website;
    /**
     * @var array
     */
    protected $ids;
    /**
     * @var array
     */
    protected $items;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    public function __construct(
        ResourceConnection $resourceConnection
    )
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return mixed|void
     */
    public function current()
    {
        $current = current($this->items);

        $category = new Category();
        $category->setId((int)$current[0]['category_id']);
        $category->setData(Category::PERMISSIONS,$current);

        return $category;
    }


    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    /**
     * @return bool|float|int|mixed|string|null
     */
    public function key()
    {
        return current($this->items);
    }

    /**
     * @return bool|mixed
     */
    public function valid()
    {
        return current($this->items);
    }

    /**
     *
     */
    public function rewind()
    {
        $permissionsByCategory = [];
        $select = $this->resourceConnection->getConnection()
            ->select()
            ->from(
                'magento_catalogpermissions'
            );
        $result = $this->resourceConnection->getConnection()->fetchAll($select, [], Zend_Db::FETCH_ASSOC);
        foreach ($result as $permission) {
            if ($permission['website_id'] == $this->website || $permission['website_id'] == null) {
                $permissionsByCategory[(int)$permission['category_id']][] = $permission;
            }
        }
        $this->items = $permissionsByCategory;
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->website = $store->getWebsiteId();
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        $this->website = $website->getId();
        return $this;
    }
}
